from colout2.colout import ThemeEntry


def theme(context):
    return context, [
        # traceback header
        ThemeEntry("^Traceback .*$", "blue"),
        # File, line, in
        ThemeEntry(
            "^\s{2}(File \")(/*.*?/)*([^/:]+)(\", line) ([0-9]+)(, in) (.*)$",
            "blue,  none,  white,blue,  yellow,blue",
            "normal,normal,bold, normal,normal,bold"
        ),
        # ThemeEntry("^\s{2}File \"(.*)\", line ([0-9]+), in (.*)$", "white,yellow,white", "normal,normal,bold" ),
        # Error name
        ThemeEntry("^([A-Za-z]*Error):*", "red", "bold"),
        ThemeEntry("^([A-Za-z]*Exception):*", "red", "bold"),
        # any quoted things
        ThemeEntry("Error.*['\"](.*)['\"]", "magenta"),
        # python code
        ThemeEntry("^\s{4}.*$", "Python", "monokai"),
    ]



from colout2.colout import ThemeEntry


def theme(context):
    # CMake theme:
    #  actions performing in cyan
    performing = "cyan"
    #  actions performed in green
    performed = "green"
    #  actions taking an unknown time
    untimed = "blue"

    # If the user do not ask for his own colormap
    if not context["user_defined_colormaps"]:
        # A palette that goes: purple, orange, white
        percs = [45, 39, 33, 27, 21, 57, 63, 62, 98, 97, 133, 132, 138, 173, 172, 208, 214, 220, 226, 228, 229, 230,
                 231, 255]
        context["colormaps"]["Scale"] = percs

    return context, [
        # Configure...
        ThemeEntry("^--.*works", performed),
        ThemeEntry("^--.*done", performed),
        ThemeEntry("^-- Found.*NO", "red"),
        ThemeEntry("^-- Found.*", performed),
        ThemeEntry("^--.*broken", "red"),
        ThemeEntry("^-- Coult NOT find.*", "red"),
        ThemeEntry("^-- Configuring incomplete, errors occurred!", "red"),
        ThemeEntry("^--.*", performing),
        # Errors
        ThemeEntry("CMake Error:", "red"),
        ThemeEntry("CMake Warning", "yellow"),
        # Scan
        ThemeEntry("^(Scanning dependencies of target)(.*)$",
         performing, "normal,bold"),
        # Link (make)
        ThemeEntry("^(Linking .* )(library|executable) (.*/)*(.+(\.[aso]+)*)$",
         untimed, "normal,normal,bold"),
        # [percent] Creating something
        ThemeEntry("^\[\s*[0-9/]+%?\]\s(.*Creating.*)$",
         performing, "normal"),
        # [percent] Built
        ThemeEntry("^\[\s*[0-9/]+%?\]\s(Built target)(\s.*)$",
         performed, "normal,bold"),
        # [percent] Building
        ThemeEntry("^\[\s*[0-9/]+%?\]\s(Building \w* object)\s+(.*)(\.dir)(.*/)([-\w]+).c.*.o$",
         performing + "," + performing + "," + performing + ",Hash," + performing, "normal,normal,normal,normal,bold"),
        # [percent] Generating
        ThemeEntry("^\[\s*[0-9/]+%?\]\s(Generating)(\s+.*)$",
         performing, "normal,bold"),
        # make errors
        ThemeEntry("make\[[0-9]+\].*", "yellow"),
        ThemeEntry("(make: \*\*\* \[.+\] )(.* [0-9]+)", "red", "normal,bold"),
        # progress percentage (make)
        ThemeEntry("^(\[\s*[0-9]+%\])", "Scale")
    ]



# encoding: utf-8
from colout2.colout import ThemeEntry


def default_gettext(msg):
    return msg


def theme(context):
    import os
    import gettext
    import locale

    section = "blue"

    # get g++ version
    gv = os.popen("g++ -dumpversion").read().strip()

    # get the current translations of gcc
    try:
        t = gettext.translation("gcc-" + gv)
    except IOError:
        _ = default_gettext
    else:
        _ = t.gettext
    # _("msg") will return the given message, translated

    # if the locale is unicode
    enc = locale.getpreferredencoding()
    if "UTF" in enc:
        # gcc will use unicode quotes
        qo = "[‘`]"
        qc = "[’']"
    else:
        # rather than ascii ones
        qo = "['`]"
        qc = "'"

    return context, [
        # Command line
        ThemeEntry("[/\s]([cg]\+\+-*[0-9]*\.*[0-9]*)", "white", "bold"),
        ThemeEntry("\s(\-D)(\s*[^\s]+\s)", "none,green", "normal,bold"),
        ThemeEntry("\s-g\s", "green", "normal"),
        ThemeEntry("\s-O[0-4]*\s", "green", "normal"),
        ThemeEntry("\s-[Wf][^\s]*", "magenta", "normal"),
        ThemeEntry("\s(-I)(/*[^\s]+/)([^/\s]+)", "none,blue", "normal,normal,bold"),
        ThemeEntry("\s(-L)(/*[^\s]+/)([^/\s]+)", "none,cyan", "normal,normal,bold"),
        ThemeEntry("\s(-l)([^/\s]+)", "none,cyan", "normal,bold"),
        ThemeEntry("\s-[oc]", "red", "bold"),
        ThemeEntry("\s(-+std)=*([^s]+)", "red", "normal,bold"),

        # Important messages
        ThemeEntry(_("error: "), "red", "bold"),
        ThemeEntry(_("fatal error: "), "red", "bold"),
        ThemeEntry(_("warning: "), "magenta", "bold"),
        ThemeEntry(_("undefined reference to "), "red", "bold"),
        # [-Wflag]
        ThemeEntry("\[-W.*\]", "magenta"),

        # Highlight message start:
        #   path   file   ext     : line   :  col     …
        ThemeEntry("(/.*?)/([^/:]+): (In .*)" + qo,
         section,
         "normal,normal,bold"),

        ThemeEntry("(/.*?)/([^/:]+): (At .*)",
         section,
         "normal,normal,bold"),

        ThemeEntry(_("In file included from"), section),

        # Highlight locations:
        #   path   file   ext     : line   :  col     …
        ThemeEntry("(/.*?)/([^/:]+):([0-9]+):*([0-9]*)(.*)",
         "none,white,yellow,none,none",
         "normal,normal,normal,normal"),

        # source code in single quotes
        ThemeEntry(qo + r"(.*?)" + qc, "Cpp", "monokai"),

        # source code after a "note: candidate are/is:"
        ThemeEntry(_("note: ") + r"((?!.*(" + qo + "|" + qc + ")).*)$", "Cpp", "monokai"),
        # [ _("note: ")+"(candidate:)(.*)$", "green,Cpp", "normal,monokai" ),
        # after the code part, to avoid matching ANSI escape chars
        ThemeEntry(_("note: "), "green", "normal")
    ]



from colout2 import colout_configure
from colout2.colout import ThemeEntry


def theme(context):
    context, th = colout_configure.theme(context)
    th.extend([
        ThemeEntry("error: ", "red", "bold"),
        ThemeEntry("warning: ", "magenta", "bold"),
        ThemeEntry(r"^[^\s:]*:[0-9:]*", "blue", "bold"),
        ThemeEntry(r"AC_[A-Z_]+", "cyan"),
        ThemeEntry(r"ACLOCAL_[A-Z_]+", "cyan"),
        ThemeEntry(r"'.*'[\s|.]", "yellow"),
        ThemeEntry(r"\".*\"\s", "yellow")
    ])
    return context, th


from colout2.colout import ThemeEntry


def theme(context):
    return context, [
        # LaTeX
        ThemeEntry("This is .*TeX.*$", "white", "bold"),
        ThemeEntry("(LaTeX Warning): (.*) `(.*)' on page [0-9] (.*) on input line [0-9]+.$",
         "magenta,magenta,white,magenta", "normal,bold,normal"),
        ThemeEntry("(LaTeX Warning): (.*)", "magenta", "normal,bold"),
        ThemeEntry("(LaTeX Error): (.*)", "red", "normal,bold"),
        ThemeEntry("^(.*\.tex):([0-9]+): (.*)", "white,yellow,red", "normal,normal,bold"),
        # ThemeEntry("on (page [0-9]+)", "yellow", "normal" ),
        ThemeEntry("on input (line [0-9]+)", "yellow", "normal"),
        ThemeEntry("^! .*$", "red", "bold"),
        ThemeEntry("(.*erfull) ([^\s]+).* in [^\s]+ at (lines [0-9]+--[0-9]+)",
         "magenta,magenta,yellow", "normal"),
        ThemeEntry("\\[^\s]+\s", "white", "bold"),
        ThemeEntry("^l\.([0-9]+) (.*)", "yellow,tex"),
        ThemeEntry("^\s+(.*)", "tex"),
        ThemeEntry("(Output written on) (.*) \(([0-9]+ pages), [0-9]+ bytes\).",
         "blue,white,blue", "normal,bold,normal"),
        ThemeEntry("WARNING.*", "magenta", "normal"),
        ThemeEntry("warning.*", "magenta", "normal"),

        # BiBTeX
        ThemeEntry("^(I couldn't) (.*)", "red", "normal,bold"),
        ThemeEntry("(I found) no (.*)", "red"),
        ThemeEntry("^---(line [0-9]+) of file (.*)", "yellow,white", "normal"),
    ]



import colout_cmake
from colout2.colout import ThemeEntry


def theme(context):
    # Ninja theme

    # Inherit from the CMake theme
    context, th = colout_cmake.theme(context)

    # Because Ninja note progress as a fraction, we do not want the scale of a percentage
    context["scale"] = (0, 1)

    # Link (ninja)
    th.append(ThemeEntry("^\[[0-9/]+\]\s?(Linking .* )(library|executable) (.*/)*(.+(\.[aso]+)*)$",
               "blue", "normal,normal,bold"))
    # progress percentage (ninja)
    th.append(ThemeEntry("^(\[[0-9]+/[0-9]+\])", "Scale"))

    return context, th



import logging
import os

import sys

from pygments.formatters.terminal import TerminalFormatter
from pygments.formatters.terminal256 import Terminal256Formatter
from pygments.lexers import get_lexer_by_name

from colout2.colout import (UnknownColor, _args_parse, load_resources, load_palettes, load_themes, DuplicatedPalette,
                           write_all, set_special_colormaps, colortheme, colorup)


def main():
    from colout2.colout import context, debug, highlight
    global debug
    error_codes = {"UnknownColor": 1, "DuplicatedPalette": 2}

    usage = "A regular expression based formatter that color up an arbitrary text stream."

    #####################
    # Arguments parsing #
    #####################
    pattern, color, style, on_groups, as_colormap, as_theme, as_source, as_all, myscale, \
    debug, resources, palettes_dirs, themes_dirs, default_colormap \
        = _args_parse(sys.argv, usage)

    if debug:
        lvl = logging.DEBUG
    else:
        lvl = logging.ERROR

    logging.basicConfig(format='[colout2] %(levelname)s: %(message)s', level=lvl)

    ##################
    # Load resources #
    ##################
    try:
        # Search for available resources files (themes, palettes)
        # in the same dir as the colout2.py script
        res_dir = os.path.dirname(os.path.realpath(__file__))

        # this must be called before args parsing, because the help can list available resources
        load_resources(res_dir, res_dir)

        # try additional directories if asked
        if palettes_dirs:
            for adir in palettes_dirs:
                try:
                    os.chdir(adir)
                except OSError as e:
                    logging.warning("cannot read palettes directory %s, ignore it" % adir)
                    continue
                else:
                    load_palettes(adir)

        if themes_dirs:
            for adir in themes_dirs:
                try:
                    os.chdir(adir)
                except OSError as e:
                    logging.warning("cannot read themes directory %s, ignore it" % adir)
                    continue
                else:
                    load_themes(adir)

    except DuplicatedPalette as e:
        logging.error("duplicated palette file name: %s" % e)
        sys.exit(error_codes["DuplicatedPalette"])

    if resources:
        asked = [r.lower() for r in pattern.split(",")]

        def join_sort(l):
            """
            Sort the given list in lexicographical order,
            with upper-cases first, then lower cases
            join the list with a comma.

            >>> join_sort(["a","B","A","b"])
            'A, a, B, b'
            """
            return ", ".join(sorted(l, key=lambda s: s.lower() + s))

        # print("Available resources:")
        for res in asked:
            if "style" in res or "all" in res:
                print("STYLES: %s" % join_sort(context["styles"]))

            if "color" in res or "all" in res:
                print("COLORS: %s" % join_sort(context["colors"]))

            if "special" in res or "all" in res:
                print("SPECIAL: %s" % join_sort(["random", "Random", "scale", "Scale", "hash", "Hash", "colormap"]))

            if "theme" in res or "all" in res:
                if len(context["themes"]) > 0:
                    print("THEMES: %s" % join_sort(context["themes"].keys()))
                else:
                    print("NO THEME")

            if "colormap" in res or "all" in res:
                if len(context["colormaps"]) > 0:
                    print("COLORMAPS: %s" % join_sort(context["colormaps"]))
                else:
                    print("NO COLORMAPS")

            if "lexer" in res or "all" in res:
                if len(context["lexers"]) > 0:
                    print("SYNTAX COLORING: %s" % join_sort(context["lexers"]))
                else:
                    print("NO SYNTAX COLORING (check that python3-pygments is installed)")

        sys.exit(0)  # not an error, we asked for help

    ############
    # Coloring #
    ############

    try:
        if myscale:
            context["scale"] = tuple([float(i) for i in myscale.split(",")])
            logging.debug("user-defined scale: %f,%f" % context["scale"])

        # Default color maps
        if default_colormap:
            if default_colormap not in context["colormaps"]:
                cmap = default_colormap.split(",")

            elif default_colormap in context["colormaps"]:
                cmap = context["colormaps"][default_colormap]

            set_special_colormaps(cmap)

        # explicit color map
        if as_colormap is True and color not in context["colormaps"]:
            context["colormaps"]["Default"] = color.split(",")  # replace the colormap by the given colors
            context["colormaps"]["default"] = color.split(",")  # replace the colormap by the given colors
            color = "colormap"  # use the keyword to switch to colormap instead of list of colors
            logging.debug("used-defined default colormap: %s" % ",".join(context["colormaps"]["Default"]))

        # if theme
        if as_theme:
            logging.debug("asked for theme: %s" % pattern)
            assert (pattern in context["themes"].keys())
            context, theme = context["themes"][pattern].theme(context)
            write_all(as_all, sys.stdin, sys.stdout, colortheme, theme)

        # if pygments
        elif as_source:
            logging.debug("asked for lexer: %s" % pattern.lower())
            assert (pattern.lower() in context["lexers"])
            lexer = get_lexer_by_name(pattern.lower())
            # Python => 256 colors, python => 8 colors
            ask_256 = pattern[0].isupper()
            if ask_256:
                logging.debug("256 colors mode")
                try:
                    formatter = Terminal256Formatter(style=color)
                except:  # style not found
                    logging.warning("style %s not found, fallback to default style" % color)
                    formatter = Terminal256Formatter()
            else:
                logging.debug("8 colors mode")
                formatter = TerminalFormatter()

            write_all(as_all, sys.stdin, sys.stdout, highlight, lexer, formatter)

        # if color
        else:
            write_all(as_all, sys.stdin, sys.stdout, colorup, pattern, color, style, on_groups)

    except UnknownColor as e:
        if debug:
            import traceback
            for var in context:
                print(var, context[var])
            print(traceback.format_exc())
        logging.error("unknown color: %s (maybe you forgot to install python3-pygments?)" % e)
        sys.exit(error_codes["UnknownColor"])



from colout2.colout import ThemeEntry


def theme(context):
    # This theme expect a formatted JSON input, with items spread across lines.
    # See tools like "python -m json.tool" or "json_xs"
    return context, [
        ThemeEntry('[\[\]{}],*\s*\n'),
        ThemeEntry("\" (:) ", "yellow"),
        ThemeEntry("[\]}\"](,)", "yellow"),
        ThemeEntry("\"(-*[0-9]+\.*[0-9]*e*-*[0-9]*)\"", "blue"),
        ThemeEntry("\"(.*)\"", "green"),
        ThemeEntry("""["']""", "cyan")
    ]



# encoding: utf-8
from colout2.colout import ThemeEntry


def theme(context):
    return context, [
        ThemeEntry("^(checking .*)(yes|found|ok)$", "green", "normal,bold"),
        ThemeEntry("^(checking .*)(no|none)$", "yellow", "normal,bold"),
        ThemeEntry("^(configure:) (error:)(.*)", "red", "normal,bold"),
        ThemeEntry("^(configure:)(.*)", "magenta", "normal,bold"),
        ThemeEntry("^(checking .*)", "blue", ""),
        ThemeEntry("^(config.status:) (creating )(.*)", "cyan,blue", "normal,normal,bold"),
        ThemeEntry("^(config.status:) (executing )(.*)", "cyan,green", "normal,normal,bold"),
    ]



# encoding: utf-8
from colout2.colout import ThemeEntry


def theme(context):
    style = "monokai"
    return context, [
        ThemeEntry("^(.*\.java):([0-9]+):\s*(warning:.*)$", "white,yellow,magenta", "normal,normal,bold"),
        ThemeEntry("^(.*\.java):([0-9]+):(.*)$", "white,yellow,red", "normal,normal,bold"),
        ThemeEntry("^(symbol|location)\s*:\s*(.*)$", "blue,Java", "bold," + style),
        ThemeEntry("^(found)\s*:\s*(.*)", "red,Java", "bold," + style),
        ThemeEntry("^(required)\s*:\s*(.*)", "green,Java", "bold," + style),
        ThemeEntry("^\s*\^$", "cyan", "bold"),
        ThemeEntry("^\s+.*$", "Java", style),
        ThemeEntry("[0-9]+ error[s]*", "red", "bold"),
        ThemeEntry("[0-9]+ warning[s]*", "magenta", "bold"),
    ]



from colout2.colout import ThemeEntry


def theme(context):
    p = "([rwxs-])"
    reg = "^([d-])" + p * 9 + "\s.*$"
    colors = "blue" + ",green" * 3 + ",yellow" * 3 + ",red" * 3
    styles = "normal" + ",normal,italic,bold" * 3
    return context, [ThemeEntry(reg, colors, styles)]



#!/usr/bin/env python3

import os
import sys

import colout2

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python3 setup.py sdist upload')
    sys.exit()

packages = ['colout2']

requires = ['argparse', 'pygments', 'babel']

setup(
    name='colout2',
    version='1.0.1',
    description='Color Up Arbitrary Command Output.',
    long_description=open('README.md').read(),
    author='Omer Katz',
    author_email='omer.drow@gmail.com',
    url='http://nojhan.github.com/colout/',
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'colout2': 'colout2'},
    include_package_data=True,
    install_requires=requires,
    license=open('LICENSE').read(),
    entry_points={
        'console_scripts': [
            'colout2 = colout2.__main__:main'
        ]
    },
    zip_safe=False,
)



from colout2.colout import ThemeEntry


def theme(context):
    # CTest theme:
    passed = "green"
    notpassed = "red"

    # If the user do not ask for his own colormap
    # if not context["user_defined_colormaps"]:
    #     # A palette that goes: purple, orange, white
    #     percs = [45, 39, 33, 27, 21, 57, 63, 62, 98, 97, 133, 132, 138, 173, 172, 208, 214, 220, 226, 228, 229, 230, 231, 255]
    #     context["colormaps"]["Scale"] = percs

    return context, [
        # Passed
        ThemeEntry("^\s*[0-9]+/[0-9]+ Test\s+#[0-9]+: (.*)\s+\.+\s+(Passed)", passed),
        ThemeEntry("^\s*[0-9]+/[0-9]+ Test\s+#[0-9]+: (.*)\s+\.+(\*{3}.*)\s+.*", notpassed)
    ]



# encoding: utf-8
from colout2.colout import ThemeEntry


def theme(context):
    return context, [
        # section title
        ThemeEntry("^(==[0-9]+==\s{1})(Memcheck|Copyright|Using)(.*)$", "blue", ""),
        ThemeEntry("^(==[0-9]+==\s{1})(Warning)(.*)$", "magenta", ""),
        ThemeEntry("^(==[0-9]+==\s{1}Command: )(\S*)(.*)$", "green,white", "normal,bold,normal"),
        ThemeEntry("^(==[0-9]+==\s{1})(HEAP SUMMARY:)(.*)$", "green", ""),
        ThemeEntry("^(==[0-9]+==\s{1})(All heap blocks were freed)(.*)$", "green", ""),
        ThemeEntry("^(==[0-9]+==\s{1})(.*[rR]erun.*)$", "blue", ""),
        ThemeEntry("^(==[0-9]+==\s{1})(Use --.*)$", "blue", ""),
        ThemeEntry("^(==[0-9]+==\s{1}\S+.*)$", "red", ""),
        # section explanation
        ThemeEntry("^==[0-9]+==\s{2}(\S+.*)$", "orange", ""),
        # locations adresses
        ThemeEntry("^==[0-9]+==\s{4}([atby]{2}) (0x0): (\?{3})",
         "blue,yellow,red", "normal,normal,bold"),
        ThemeEntry("^==[0-9]+==\s{4}([atby]{2}) (0x)([^:]*:) (\S+)",
         "blue,blue,blue,none", "normal"),
        # locations: library
        ThemeEntry("\(in (.*)\)", "cyan", "normal"),
        # locations: file
        ThemeEntry("\(([^\.]*\.[^:]+):([0-9]+)\)", "white,yellow", "bold,normal"),
        # leak summary
        ThemeEntry("^==[0-9]+==\s{4}(definitely lost): .* (in) .*", "red", "bold"),
        ThemeEntry("^==[0-9]+==\s{4}(indirectly lost): .* (in) .*", "orange", "bold"),
        ThemeEntry("^==[0-9]+==\s{6}(possibly lost): .* (in) .*", "yellow", "bold"),
        ThemeEntry("^==[0-9]+==\s{4}(still reachable): .* (in) .*", "green", "bold"),
        ThemeEntry("^==[0-9]+==\s{9}(suppressed): .* (in) .*", "cyan", "bold"),
    ]
